/****************************************************************************

    PROGRAM:    mailscan

    FILE:       mailcsan.c -> mailscan.exe

    Author:	Hans-Georg Franke (hansgeorgfranke@yahoo.de)

    This file is part of OpenMailScan.

    OpenMailScan is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    OpenMailScan is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with MailScan (see license.txt); if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


****************************************************************************/

#if defined(LINUX)
extern "C" {
#endif

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

/* Lotus C API for Domino and Notes include files */

#include <stdnames.h>
#include <global.h>
#include <addin.h>
#include <nsfdb.h>
#include <nif.h>
#include <nsfnote.h>
#include <nsfsearc.h>
#include <idtable.h>
#include <editods.h>
#include <osenv.h>
#include <ostime.h>
#include <osfile.h>
#include <osmisc.h>
#include <osmem.h>
#include <mailserv.h>
#include <mail.h>

#include "addin.h"
#include "log.h"
#include "config.h"
#include "processNote.h"

STATUS LNPUBLIC AddIDUnique(void far * phNoteIDTable,SEARCH_MATCH far *pSearchMatch,ITEM_TABLE far *summary_info);
STATUS LNPUBLIC Processnote (void far * phDB, DWORD NoteID);


//Global Variables

DBHANDLE	hlocdb;


/************************************************************************

    FUNCTION:   AddInMain

    PURPOSE:    Main entry point for all Lotus Domino Server add-in tasks

*************************************************************************/

STATUS LNPUBLIC  AddInMain (HMODULE hModule, int argc, char *argv[])
{

    // Local data.

    HANDLE		hOldStatusLine;  // handle to initial default status line
	HANDLE		hStatusLineDesc; // handle to new default status line
    HMODULE		hMod;            // add-in task's module handle
    int			l;
    int			n;
    STATUS		serror;
    char 		szBuffer[512];
    HANDLE 		hNoteIDTable;
    char		Version[]="0.99c";

    LogMessageText("MailScan-Addin: Startup......", NOERROR);

    AddInQueryDefaults (&hMod, &hOldStatusLine);

    AddInDeleteStatusLine (hOldStatusLine);

    hStatusLineDesc = AddInCreateStatusLine("Mailscan");

    AddInSetDefaults (hMod, hStatusLineDesc);

    AddInSetStatusText("Initializing");


    sprintf(szBuffer, "MailScan-Addin: OpenMailScan Version %s, Copyright (C) 2004 Hans-Georg Franke", Version);
	LogMessageTextCon(szBuffer, NOERROR);
	sprintf(szBuffer, "MailScan-Addin: OpenMailScan comes with ABSOLUTELY NO WARRANTY;");
	LogMessageTextCon(szBuffer, NOERROR);
	sprintf(szBuffer, "MailScan-Addin: This is free software, and you are welcome to redistribute it under certain conditions;");
	LogMessageTextCon(szBuffer, NOERROR);
	sprintf(szBuffer, "MailScan-Addin: For details see included file \"license.txt\"");
	LogMessageTextCon(szBuffer, NOERROR);


	DebugLevel=OSGetEnvironmentInt("MailScan_DebugLevel");
 	sprintf(szBuffer, "MailScan-Addin: DebugLevel: %i", DebugLevel);
	LogMessageTextCon(szBuffer, NOERROR);

	//Laden der Konfiguration aus der NOTES.INI (via config.c-loadConfig())
	loadConfig();

    	LogMessageText("MailScan-Addin: Searching for Mailboxes.", NOERROR);

	searchMailboxes();

    sprintf(szBuffer, "MailScan-Addin: Startup Completed.");
    LogMessageTextCon(szBuffer, NOERROR);

    while (!AddInIdle())
    {

        OSPreemptOccasionally();

        if (AddInSecondsHaveElapsed(10))
        {
        AddInSetStatusText("Doing 10-second job");

		for ( n=(nrMailboxes==0?0:1) ; n<=nrMailboxes; n++)
			{
				//sprintf(szBuffer, "MailScan-Addin: Scanning mailbox nr. %i ", n);
				//LogMessageTextSeverity(szBuffer, NOERROR,3);
				serror = IDCreateTable(sizeof(NOTEID), &hNoteIDTable);
				if (serror != NOERROR )
				{
					for (n=(nrMailboxes==0?0:1); nrMailboxes; n++)
					{
						if (hdb[n] != NULLHANDLE)
						{
							NSFDbClose(hdb[n]);
							hdb[n] = NULLHANDLE;
						}
					}

					return (ERR(serror));
				}

				//sprintf(szBuffer,"MailScan-Addin: Searching mailbox nr. %i", n);
				//LogMessageTextSeverity(szBuffer, NOERROR,3);

				hlocdb = hdb[n];

				serror =NSFSearch (
						hdb[n],				// database handle
						NULLHANDLE,			// selection formula (select all notes)
						NULL,				// title of view in selection formula
						0,					// search flags
						NOTE_CLASS_DATA,	// note class to find
						NULL,				// starting date (unused)
						AddIDUnique,		// call for each note found
						&hNoteIDTable,		// argument to AddIDUnique
						NULL);				// returned ending date (unused)

				if ( serror != NOERROR)
				{
					IDDestroyTable(hNoteIDTable);
					for (l=(nrMailboxes==0?0:1); l<=nrMailboxes; l++)
    					{
				 		if (hdb[l] != NULLHANDLE)
    						{
	    						NSFDbClose(hdb[l]);
	    						hdb[l] = NULLHANDLE;
						}
    					}
					sprintf(szBuffer,"MailScan-Addin: NSFSearch-Error with Mailbox Nr. %i", n);
					LogMessageTextConSeverity(szBuffer, NOERROR,1);
					return (ERR(serror));
				}

				//sprintf(szBuffer,"MailScan-Addin: Enumerate mailbox nr. %i", n);
				//LogMessageTextSeverity(szBuffer, NOERROR,3);

				if (serror = IDEnumerate(hNoteIDTable,
							Processnote,    // called for each ID
							&(hdb[n]) ))       // arg passed to func
				{
					char error_text[200];

					sprintf(szBuffer,"MailScan-Addin: IDEnumerate-Error with Mailbox Nr. %i", n);
					LogMessageTextConSeverity(szBuffer, NOERROR,0);

					OSLoadString(NULLHANDLE, ERR(serror), error_text, sizeof(error_text));
					sprintf(szBuffer,"MailScan-Addin: IDEnumerate failurereason %s", error_text);
					LogMessageTextConSeverity(szBuffer, NOERROR,0);

					IDDestroyTable(hNoteIDTable);
					for (l=(nrMailboxes==0?0:1); l<=nrMailboxes; l++)
   					{
				 		if (hdb[l] != NULLHANDLE)
   						{
    						NSFDbClose(hdb[l]);
    						hdb[l] = NULLHANDLE;
						}
   					}

					//sprintf(szBuffer,"MailScan-Addin: Exiting ..............");
					sprintf(szBuffer,"MailScan-Addin: Exiting this execution..............");
					LogMessageTextConSeverity(szBuffer, NOERROR,0);
					//return (ERR(serror));
					return (NOERROR);
				}

				//sprintf(szBuffer,"MailScan-Addin: IDDestroy for mailbox nr. %i", n);
				//LogMessageTextSeverity(szBuffer, NOERROR,3);

				serror = IDDestroyTable(hNoteIDTable);
				if ( serror != NOERROR)
				{
					char error_text[200];

					sprintf(szBuffer,"MailScan-Addin: IDDestroy-Error with Mailbox Nr. %i", n);
					LogMessageTextConSeverity(szBuffer, NOERROR,0);

					OSLoadString(NULLHANDLE, ERR(serror), error_text, sizeof(error_text));
					sprintf(szBuffer,"MailScan-Addin: IDDestroy failurereason %s", error_text);
					LogMessageTextConSeverity(szBuffer, NOERROR,0);

					for (l=(nrMailboxes==0?0:1); l<=nrMailboxes; l++)
   					{
			 			if (hdb[l] != NULLHANDLE)
   						{
    						NSFDbClose(hdb[l]);
    						hdb[l] = NULLHANDLE;
						}
   					}
					//sprintf(szBuffer,"MailScan-Addin: Exiting ..............");
					sprintf(szBuffer,"MailScan-Addin: Exiting this execution..............");
					LogMessageTextConSeverity(szBuffer, NOERROR,0);
					//return (ERR(serror));
					return (NOERROR);
				}

			}
	     }

		AddInSetStatusText("Idle");

    }   // End of main task loop.

   //	We get here when the server notifies us that it is time to terminate.
   //   This is usually when the user has entered "quit" to the server console.
   //   Clean up anything we have been doing.
   //
    AddInSetStatusText("Terminating");
	for (n=(nrMailboxes==0?0:1); n<=nrMailboxes; n++)
	{
		if (hdb[n] != NULLHANDLE)
		{
			NSFDbClose(hdb[n]);
			hdb[n] = NULLHANDLE;
		}
	}

    LogMessageTextCon("MailScan-Addin: Termination complete.", NOERROR);

   // End of add-in task.  We must "return" here rather than "exit".

    return (NOERROR);
}



STATUS LNPUBLIC AddIDUnique
            (void far * phNoteIDTable,
            SEARCH_MATCH far *pSearchMatch,
            ITEM_TABLE far *summary_info)
{
    SEARCH_MATCH SearchMatch;
    HANDLE       hNoteIDTable;
    STATUS       error;
    BOOL         flagOK;

	NOTEHANDLE	 hNote;
    WORD		field_len;
    char		field_text[1024];
    BOOL 		field_found;
    char 		szBuffer[512];

    memcpy( (char*)&SearchMatch, (char*)pSearchMatch, sizeof(SEARCH_MATCH) );

    if (!(SearchMatch.SERetFlags & SE_FMATCH))
        return (NOERROR);


    hNoteIDTable = *((HANDLE far *)phNoteIDTable);


    if (error = NSFNoteOpen (hlocdb, SearchMatch.ID.NoteID, 0, &hNote))
    {
		sprintf(szBuffer,"MailScan-Addin: Can't open mail in mail.box. Already routed");
		LogMessageTextConSeverity(szBuffer, NOERROR,3);
        return (NOERROR);
    }


	field_found = NSFItemIsPresent ( hNote, "MailScanNoteProcessed",(WORD) strlen ("MailScanNoteProcessed"));

	if (field_found)
	{
		field_len = NSFItemGetText ( hNote,"MailScanNoteProcessed",field_text, sizeof (field_text));

		if ( strcmp(field_text,"TRUE")==0 )
		{
		sprintf(szBuffer,"MailScan-Addin: MailScanNoteProcessed==TRUE found");
		LogMessageTextConSeverity(szBuffer, NOERROR,3);
		return (NOERROR);
		}
	}

	NSFNoteClose(hNote);


    if (error = IDInsert(hNoteIDTable, SearchMatch.ID.NoteID, &flagOK))
    {
		sprintf(szBuffer,"MailScan-Addin: Error adding NotID to NoteIDTable");
		LogMessageTextConSeverity(szBuffer, NOERROR,3);
        return (ERR(error));
    }

    return (NOERROR);
}


#if defined(LINUX)
}
#endif

