/*
    PROGRAM:    mailscan

    FILE:       config.c

    Author:	Hans-Georg Franke (hansgeorgfranke@yahoo.de)

    This file is part of OpenMailScan.

    OpenMailScan is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    OpenMailScan is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with MailScan (see license.txt); if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
    */
    
#if defined(LINUX)
extern "C" {
#endif
#include <stdio.h>
#include <string.h>
#include <stdlib.h>


#include "config.h"
#include "log.h"

DBHANDLE	hdb[10];
WORD		nrMailboxes=10;

char	DATADIR[MAXPATH];
char	sconfigview[256]="configuration";
char    sconfigdatabase[MAXPATH]="virconf.nsf";
char    environdatabase[MAXPATH]="virconf.nsf";
char	ssearchconfigure[256]="Konfiguration";
char	workdir[MAXPATH]="MailScan";
char	virscanner[256];
char	virparm[256];
BOOL scandirectory=FALSE;

LONG rc[PARAMCOUNT +1];
char virscan[PARAMCOUNT +1][256];
BOOL isbefore[PARAMCOUNT +1];
int	 removefile[PARAMCOUNT +1];
char removetext[PARAMCOUNT +1 ][256];
char removemail[PARAMCOUNT +1 ][256];
char bodytext[PARAMCOUNT +1][256];

void loadConfig(void)
{
	BOOL		retcode;
	STATUS		sStatus=NOERROR;
	HANDLE		hconfigdb=NULLHANDLE;
    NOTEID      nidView=0;
    HCOLLECTION hCollection=NULLHCOLLECTION;
    COLLECTIONPOSITION    cp;
    DWORD       dwFound;
    HANDLE		hBuffer = NULLHANDLE;
    WORD    	wSignalFlags;
    NOTEID		*pnid;
    NOTEID		nid;
    NOTEHANDLE	nHandle;
	char 		szBuffer[512];
	char		tmpBuffer[512];
	int			n;
	int			x;

	x = OSGetDataDirectory(DATADIR);

	retcode=OSGetEnvironmentString("MailScanConfigDB", environdatabase, sizeof(environdatabase) -1 );

	if (retcode)
	{
		strcpy(sconfigdatabase, environdatabase);
	}

    /* Read the configuration data from config-database */
    sStatus = NSFDbOpen(sconfigdatabase, &hconfigdb);

    	if (sStatus != NOERROR)
    	{
		goto noConfig;
	}

	sStatus = NIFFindView(hconfigdb, sconfigview, &nidView);

	if (sStatus != NOERROR)
	{
		goto noConfig;
	}

	sStatus = NIFOpenCollection(hconfigdb,
                                hconfigdb,
                                nidView,
                                0,
                                NULLHANDLE,
                                &hCollection,
                                NULL,
                                NULL,
                                NULL,
                                NULL);

    	if (sStatus != NOERROR)
    	{
		goto noConfig;
	}

	sStatus = NIFUpdateCollection(hCollection);



    	if (sStatus != NOERROR)
    	{
		goto noConfig;
	}

	sStatus = NIFFindByName(hCollection, ssearchconfigure, FIND_PARTIAL, &cp, &dwFound);


	/*Dok-handel holen */

	if (sStatus != NOERROR)
	{
		goto noConfig;
	}

	do
    	{
        	sStatus = NIFReadEntries(hCollection,
                                 &cp,
                                 NAVIGATE_CURRENT,
                                 0L,
                                 NAVIGATE_NEXT,
                                 0xFFFFFFFF,
                                 READ_MASK_NOTEID,
                                 &hBuffer,
                                 NULL,
                                 NULL,
                                 &dwFound,
                                 &wSignalFlags);

		if (dwFound == 0)
		{
			// No config-dok found
			LogMessageText("MailScan-Addin: No config-doc found", NOERROR);
			hBuffer=NULLHANDLE;
		}
		else if (dwFound != 1)
		{
			// Error more than one config-doks
			LogMessageText("MailScan-Addin: More than one config-doc found", NOERROR);
			hBuffer=NULLHANDLE;
		}
        	if ( (sStatus == NOERROR) & (hBuffer != NULLHANDLE) )
       	 	{
        	    	pnid = (NOTEID *)OSLockObject(hBuffer);
            		if (pnid != (NOTEID *)NULL)
            		{
                		sStatus = NSFNoteOpen(hconfigdb, pnid[0], 0, &nHandle);
                	if (sStatus == NOERROR)
                	{

				//VirScanCmd
				memset(szBuffer, '\0', sizeof(szBuffer));
                NSFNoteGetInfo(nHandle, _NOTE_ID, &nid);
				NSFItemGetText ( nHandle,"VirScanCmd",virscanner, sizeof (virscanner));
				sprintf(szBuffer, "MailScan-Addin: Calling scanner with \"%s\"", virscanner);
				LogMessageTextSeverity(szBuffer, NOERROR, 1);

				//VirScanParm
				memset(szBuffer, '\0', sizeof(szBuffer));
				NSFItemGetText ( nHandle,"VirScanParm",virparm, sizeof (virparm));

				//ScanDirectory
				memset(szBuffer, '\0', sizeof(szBuffer));
				NSFItemGetText ( nHandle,"scandir",szBuffer, sizeof (szBuffer));
				if (strcmp(szBuffer, "File") != 0)
				{
					scandirectory = TRUE;
					sprintf(szBuffer, "MailScan-Addin: Calling with Param \"%s [DIRECTROY]\\\"", virparm);
				}
				else
				{
					scandirectory = FALSE;
					sprintf(szBuffer, "MailScan-Addin: Calling with Param \"%s [FULLFILENAME]\"", virparm);
				}
				LogMessageTextSeverity(szBuffer, NOERROR,1);

				//workind girectory
				memset(szBuffer, '\0', sizeof(szBuffer));
				NSFItemGetText ( nHandle,"workingdir",workdir, sizeof (workdir));
				if (strcmp(workdir,"") == 0)
				{
					strcpy(workdir,"MailScan");
				}
				sprintf(szBuffer, "MailScan-Addin: Working directory %s", workdir);
				LogMessageTextSeverity(szBuffer, NOERROR,1);

				for (n=0; n<PARAMCOUNT; n++)
				{
					sprintf(szBuffer,"rc%i",n);
					rc[n] = NSFItemGetLong ( nHandle,szBuffer,rc[n]);

					sprintf(szBuffer,"before%i",n);
					NSFItemGetText ( nHandle,szBuffer,szBuffer, sizeof (szBuffer));
					if ( !strcmp(szBuffer,"") || !strcmp(szBuffer,"1"))
						isbefore[n] = TRUE;
					else
						isbefore[n] = FALSE;
					
					sprintf(szBuffer,"deletefile%i",n);
					NSFItemGetText ( nHandle,szBuffer,tmpBuffer, sizeof (tmpBuffer));

					removefile[n] = 1;
					if (strncmp(tmpBuffer,"0",1)==0)
						removefile[n] = 0;
					if (strncmp(tmpBuffer,"2",1)==0)
						removefile[n] = 2;

					sprintf(szBuffer,"removetext%i",n);
					NSFItemGetText ( nHandle,szBuffer,removetext[n], sizeof (removetext[n]));
					if (strcmp(removetext[n],"") == 0 )
					{
						sprintf(removetext[n],"Suspicous attachment removed: ");
					}

					sprintf(szBuffer,"removemail%i",n);
					NSFItemGetText ( nHandle,szBuffer,removemail[n], sizeof (removemail[n]));
					if (strcmp(removemail[n],"") == 0 )
					{
						sprintf(removemail[n],"noone@nowhere");
					}

					sprintf(szBuffer,"VirScan%i",n);
					NSFItemGetText ( nHandle,szBuffer,virscan[n], sizeof (virscan[n]));

					sprintf(szBuffer,"BodyText%i",n);
					NSFItemGetText ( nHandle,szBuffer,bodytext[n], sizeof (bodytext[n]));

					sprintf(szBuffer, "MailScan-Addin: ReturnCode%i: %d - Text: \"%s\"", n, rc[n], virscan[n]);
					LogMessageTextSeverity(szBuffer, NOERROR,1);
					sprintf(szBuffer, "MailScan-Addin: Handling before%i: %i", n, isbefore[n]);
					LogMessageTextSeverity(szBuffer, NOERROR,1);
					sprintf(szBuffer, "MailScan-Addin: Deleting file with rc%i: %i", n, removefile[n]);
					LogMessageTextSeverity(szBuffer, NOERROR,1);
					sprintf(szBuffer, "MailScan-Addin: Replacing text for rc%i: %s", n, removetext[n]);
					LogMessageTextSeverity(szBuffer, NOERROR,1);
					sprintf(szBuffer, "MailScan-Addin: Send mail for rc%i: %s", n, removemail[n]);
					LogMessageTextSeverity(szBuffer, NOERROR,1);
				}

				//VirScanError
				NSFItemGetText ( nHandle,"VirScanError",virscan[PARAMCOUNT], sizeof (virscan[PARAMCOUNT]));
				NSFItemGetText ( nHandle,"beforeerr",szBuffer, sizeof (szBuffer));
				if ( !strcmp(szBuffer,"") || !strcmp(szBuffer,"1"))
					isbefore[PARAMCOUNT] = TRUE;
				else
					isbefore[PARAMCOUNT] = FALSE;

				removefile[PARAMCOUNT] = NSFItemGetLong ( nHandle,"deletefileerr",1);

				NSFItemGetText ( nHandle,"BodyTextErr",bodytext[PARAMCOUNT], sizeof (bodytext[PARAMCOUNT]));
				sprintf(szBuffer, "MailScan-Addin: Other Configuration Error: \"%s\"", virscan[PARAMCOUNT]);
				LogMessageTextSeverity(szBuffer, NOERROR,1);

				sprintf(removetext[PARAMCOUNT],"Suspicous attachment removed: ");
				sprintf(szBuffer,"removeextErr",PARAMCOUNT);
				NSFItemGetText ( nHandle,szBuffer,removetext[PARAMCOUNT], sizeof (removetext[PARAMCOUNT]));

				sprintf(removemail[PARAMCOUNT],"noone@nowhere");
				sprintf(szBuffer,"removemailErr",PARAMCOUNT);
				NSFItemGetText ( nHandle,szBuffer,removemail[PARAMCOUNT], sizeof (removemail[PARAMCOUNT]));

			}
                	if (nHandle != NULLHANDLE)
                	{
                    		NSFNoteClose(nHandle);
                    		nHandle = NULLHANDLE;
                	}

                	OSUnlockObject(hBuffer);
            		}

            		OSMemFree(hBuffer);
            		hBuffer = NULLHANDLE;
        	}
    	} while (wSignalFlags & SIGNAL_MORE_TO_DO);



noConfig:

    if (sStatus != NOERROR)
	{
		setDefaults();
		LogMessageText("MailScan-Addin: Error finding configuration information. Using defaults!", NOERROR);
	}

    LogMessageText("MailScan-Addin: Initialization complete.", NOERROR);
    AddInSetStatusText("Idle");

}

void setDefaults(void)
{
	int n=0;

	#if defined(NT)
	sprintf(virscanner,"d:\\programme\\OfficeScan NT\\pcscan.exe");
	sprintf(virparm,"");
	#else
	sprintf(virscanner,"/usr/local/f-prot/f-prot");
	sprintf(virparm,"-dumb -archive -silent");
	#endif

	for (n=0; n<PARAMCOUNT; n++)
	{
		rc[n] = n;
		isbefore[n] = TRUE;
		removefile[n] = FALSE;
		sprintf(virscan[n],"(Not Set)");
		sprintf(bodytext[n], "");
		sprintf(removetext[n],"Suspicous attachment removed: ");
		sprintf(removemail[n],"noone@nowhere");
	}
	rc[PARAMCOUNT] = PARAMCOUNT;
	isbefore[PARAMCOUNT] = TRUE;
	removefile[PARAMCOUNT] = FALSE;
	sprintf(virscan[PARAMCOUNT], "(Not Set)");
	sprintf(bodytext[PARAMCOUNT], "");
	sprintf(removetext[PARAMCOUNT],"Suspicous attachment %s removed");
	sprintf(removemail[PARAMCOUNT],"noone@nowhere");

	return;
}

STATUS searchMailboxes(void)
{
    WORD		x;    
    char 		szMailServerName[MAXUSERNAME+1];
    char        szMailFileName[MAXUSERNAME+1];
    char        szMailBoxPath[MAXPATH+1];
	char 		szBuffer[512];
    int 		n;
    STATUS		serror;


	x = OSGetDataDirectory(DATADIR);

    /* Mehrere Mailboxen abprfen, Max 10! */

    for (n=0; n<=10 ; n++)
    {
		hdb[n]=NULLHANDLE;
	}

    for (n=10; n>0 ; n--)
    {

    	sprintf(szMailServerName,"");
    	sprintf(szMailFileName,"mail%i.box",n);

    	OSPathNetConstruct( NULL,
	                    szMailServerName,
	                    szMailFileName,
                            szMailBoxPath);

    	serror = NSFDbOpen(szMailBoxPath, &hdb[n]);


		if (serror != NOERROR)
		{
			nrMailboxes--;
		}
    }

    if (nrMailboxes == 0)
    {
    	strcpy(szMailServerName,"");
    	sprintf(szMailFileName,"mail.box");

    	OSPathNetConstruct( NULL,
	                    szMailServerName,
	                    szMailFileName,
                            szMailBoxPath);

    	serror = NSFDbOpen(szMailBoxPath, &hdb[0]);
	if (serror != NOERROR)
	{
		LogMessageTextSeverity("MailScan-Addin: unable to open mail.box.", serror,1);
		return(serror);
	}
    }

    if (nrMailboxes > 0)
      sprintf(szBuffer, "MailScan-Addin: Number of Mailboxes: %i.", nrMailboxes);
    else
      sprintf(szBuffer, "MailScan-Addin: No Multiple Mailboxes.", nrMailboxes);

    LogMessageText(szBuffer, serror);

	return (NOERROR);
}
#if defined(LINUX)
}
#endif
