/*
    PROGRAM:    mailscan

    FILE:       mailscan.c

    Author:	Hans-Georg Franke (hansgeorgfranke@yahoo.de)

    This file is part of OpenMailScan.

    OpenMailScan is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    OpenMailScan is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with MailScan (see license.txt); if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
    */


#if defined(LINUX)
extern "C" {
#endif

#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>

/*
 *  Domino and Notes header files
 */
#include <global.h>
#include <osmem.h>
#include <osfile.h>
#include <osenv.h>
#include <nsfdata.h>
#include <nsfdb.h>
#include <nsfnote.h>
#include <editods.h>
#include <stdnames.h>
#include <mailserv.h>
#include <mail.h>
#include <textlist.h>

#include "log.h"
#include "mailscan.h"

/*========================== GLOBAL VARIABLES =============================*/

EMHANDLER         gHandlerProc;
HEMREGISTRATION	  hHandler;
BOOL              gHooksRegistered=FALSE;
WORD              gRecursionID;
char              gTextBuffer[1024];
char              DataDir[MAXPATH+1];
char              FileName[MAXPATH+1];
char              szMailBoxPath[MAXPATH+1];
char              szString[MAXSPRINTF + 1];
char              UserName[MAXUSERNAME + 1];
char			  virscanencrypted[256];


#if defined(NT)
CRITICAL_SECTION    gCriticalSection;
#endif

/************************* End of Global Variables *************************
****************************************************************************/


/*======================= LOCAL FUNCTION PROTOTYES ========================*/

DLL_EXPORT_PREFIX STATUS LNPUBLIC DLL_EXPORT_INFIX MainEntryPoint (void);
STATUS LNPUBLIC EMHandlerProcMailScan( EMRECORD FAR * pExRecord);
STATUS DeregisterEntry();


#if defined(NT)
BOOL WINAPI DllMain( HINSTANCE hInstance, DWORD fdwReason, LPVOID lpReserved )
{
  STATUS error=NOERROR;

  switch(fdwReason)
  {
    case DLL_PROCESS_ATTACH:
      InitializeCriticalSection(&gCriticalSection);
      gHandlerProc = (EMHANDLER)MakeProcInstance((FARPROC)EMHandlerProcMailScan, hInstance);
      break;

    case DLL_PROCESS_DETACH:
      gHooksRegistered = FALSE;
      /* Free procedure instance */
      FreeProcInstance( gHandlerProc );
	  error = DeregisterEntry();
      DeleteCriticalSection(&gCriticalSection);
      break;
  }

  return( TRUE );

  UNREFERENCED_PARAMETER(lpReserved);
}

#endif




/*
* RegisterEntry: Attempts to Register one entry.
*
*   out: 0 if ok, returned error otherwise
*/
STATUS RegisterEntry()
{
    STATUS error = NOERROR;

    error = EMRegister(EM_NSFNOTEUPDATEMAILBOX,
                           EM_REG_BEFORE,
	                   (EMHANDLER)gHandlerProc,
	                   gRecursionID,
	                   &hHandler);

    return(error);
}




/*
* DeregisterEntry: Attempts to
  Deregister one entry.
*
*   out: 0 if ok, returned error otherwise
*/
STATUS DeregisterEntry()
{
    STATUS error = NOERROR;

    error = EMDeregister(hHandler);

    return(error);
}



DLL_EXPORT_PREFIX STATUS LNPUBLIC DLL_EXPORT_INFIX MainEntryPoint (void)
{
    STATUS    error = NOERROR;
	char szBuffer[256]="";

	DebugLevel=OSGetEnvironmentInt("MailScan_ExtDebugLevel");
	sprintf(szBuffer, "MailScan-ExtMgr: DebugLevel: %i", DebugLevel);
	LogMessageTextSeverity(szBuffer, NOERROR,3);

	if (OSGetEnvironmentString("MailScan_Encrypted", virscanencrypted, sizeof(virscanencrypted)-1))
	{
		sprintf(szBuffer, "MailScan-ExtMgr: Tag encrypted Mail with %s", virscanencrypted);
		LogMessageTextSeverity(szBuffer, NOERROR,3);
	}
	else
	{
		sprintf(szBuffer, "MailScan-ExtMgr: Tag for encrypted Mail not found");
		LogMessageTextSeverity(szBuffer, NOERROR,3);
		sprintf(szBuffer, "MailScan-ExtMgr: Using default Tag: \"Mail encrypted -> not scanned\"", virscanencrypted);
		sprintf(virscanencrypted, "\"Mail encrypted -> not scanned\"");
		LogMessageTextSeverity(szBuffer, NOERROR,3);
	}

	LogMessageTextSeverity("Mailscan-ExtMgr: Startup ....\n", NOERROR,3);

	#if defined (UNIX)
	strcat(FileName, "/");
	#else
	strcat(FileName, "\\");
	#endif

	/*
	*    When run on a server the dll is called multiple times. the flag
	*    keeps the main code from being executed more than once.
	*/

    if ( gHooksRegistered )
	 return(NOERROR);
	gHooksRegistered = TRUE;

    /* Get proc instance for the handler callback. */
	#if defined (NT)
	/* Done in the DllMain function at startup. */
	#else
	gHandlerProc = EMHandlerProcMailScan;
	#endif

	/*  Next get a recursion ID  for the run */

    error = EMCreateRecursionID( &gRecursionID );

    if (error == NOERROR)
       error = RegisterEntry();

    if (error != NOERROR)
		LogMessageTextSeverity("Mailscan-ExtMgr: Register Unsuccessful\n", NOERROR,3);

    LogMessageTextSeverity("Mailscan-ExtMgr: Registered",NOERROR,3);
    return( error );
}

/*=========================================================================*/





STATUS LNPUBLIC EMHandlerProcMailScan( EMRECORD FAR * pExRecord )
{
  WORD		field_len;
  char		field_text[500];


  switch (pExRecord->EId)
  {
    /* Extension Manager hook EM_MAILSENDNOTE */
    case EM_NSFNOTEUPDATEMAILBOX:
    	{
		HANDLE hNote;
		void   *pViewDesc;
		WORD   Flags;
		BOOL   *pModified;
		VARARG_PTR ap;

		char TargetText[]="Mail-Scanning Manager";

		LogMessageTextSeverity("Mailscan-ExtMgr: Starting Event\n", NOERROR,3);

		// get the arguments
		ap = pExRecord->Ap;
		hNote = VARARG_GET (ap, HANDLE);
		pViewDesc = VARARG_GET (ap, VOID *);
		Flags = VARARG_GET (ap, int);
		pModified = VARARG_GET (ap, BOOL *);

		// check error code
		if (pExRecord->Status != NOERROR)
		{
			break;
		}

		if (pExRecord->NotificationType == EM_BEFORE)
		{
			STATUS sstatus;
			BLOCKID bid;
			WORD wType;
			BLOCKID bidValue;
			DWORD dwLength;

			LogMessageTextSeverity("Mailscan-ExtMgr: Handling Event\n", NOERROR,3);
			field_len = NSFItemGetText ( hNote,"Recipients",
							field_text, sizeof (field_text));

			if ( field_len == 0)
			{
				/*Item not found*/
				break;
			}


			/*Document contains Field "MailScanNoteProcessed". Reading content*/
			field_len = NSFItemGetText ( hNote,"MailScanNoteProcessed",
							field_text, sizeof (field_text));


			if ( field_len == 0)
			{
				/*Nothing to do*/
			}
			else
			{
				if ( strncmp(field_text,"TRUE",field_len)==0 )
				{
					/*Doc is already processed*/
					break;
				}
				if ( strncmp(field_text,"RUN", field_len)==0 )
				{
					/* DocProcessing runs */
					break;
				}
			}



			NSFItemSetText(hNote, "MailScanNoteProcessed", "TRUE", 4);

			sstatus = NSFItemInfo(hNote,
						ITEM_NAME_ATTACHMENT,
						(WORD) strlen(ITEM_NAME_ATTACHMENT),
						&bid, &wType, &bidValue, &dwLength);

			if (sstatus == NOERROR)
			{
				BOOL 	issigned;
				BOOL 	isencrypted;
				char 	field_text[512];
				char 	szBuffer[700];
				BOOL 	field_found;
				STATUS	sstatus;

				sstatus = NSFNoteIsSignedOrSealed(hNote,
						&issigned, &isencrypted);

				if (isencrypted == TRUE)
				{
				  field_found = NSFItemIsPresent (hNote, "SUBJECT", (WORD) strlen ("SUBJECT"));
				  if (field_found)
				  {
					field_len = NSFItemGetText (hNote, "SUBJECT", field_text,
								sizeof (field_text));

					sprintf(szBuffer, "%s %s", field_text, virscanencrypted);
					NSFItemSetText(hNote, "SUBJECT", szBuffer, (WORD) strlen(szBuffer));
					sprintf(szBuffer, "Inhalt des Subjects: %s ", field_text);
					LogMessageTextSeverity(szBuffer, NOERROR,3);
				  }
				}
				else
				{
				  NSFItemSetText(hNote, "MailScanNoteProcessed", "Pending", 4);
				  NSFItemSetText(hNote, "RoutingState", "HOLD", 4);
				  NSFItemSetText(hNote, "FailureReason", "Processing Note by mailscan", (WORD) strlen("Processing Note by mailscan"));
				}
			}

		} // end of EM_BEFORE
		break;
	} // end of case

  } // end of switch

  //AddInLogMessageText("MailScan: Returning normally\n", NOERROR);
  return( ERR_EM_CONTINUE );

}




#if defined(LINUX)
}
#endif
